@@ -1,6 +1,7 @@
 REGEX_EMAIL = r"\b[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+[\b,]"
 ENCODINGS_TYPES = {'utf-8', 'iso8859-1'}
 USER_ENCODING = None
+STRINGS_TO_REMOVE = ['\\u200a', '\\u200d']
 DEFAULT_ENCODING = None
 PROPS_ID_MAP = {
     "0x0001": {
@@ -2176,3 +2177,25 @@ PROPS_ID_MAP = {
         "name": "AddressBookContainerId"
     }
 }
+
+KNOWN_MIME_TYPE = [
+    'composite document file v2 document',
+    'cdfv2 microsoft outlook message',
+    'mime entity, iso-8859 text',
+    'mime entity, iso-8859 text, with crlf line terminators',
+    'data',
+    'apple hfs',
+    'macintosh hfs',
+    'rfc 822 mail',
+    'smtp mail',
+
+    'multipart/signed',
+    'multipart/alternative',
+    'multipart/mixed',
+    'message/rfc822',
+
+    'application/pkcs7-mime',
+    'multipart/related',
+    '(with bom) text',
+    'ascii text',
+    'unicode text']
