@@ -1,11 +1,9 @@
-import base64
-
 import pytest
 
 from parse_emails.handle_eml import handle_eml, unfold
 from parse_emails.handle_msg import (DataModel, MsOxMessage,
                                      create_headers_map, get_msg_mail_format,
-                                     handle_msg)
+                                     handle_msg, parse_email_headers)
 from parse_emails.parse_emails import EmailParser
 
 
@@ -52,9 +50,54 @@ def test_msg_with_attachments():
     assert results.parsed_email['Attachments'] == 'dummy-attachment.txt'
 
 
-def test_eml_smtp_type():
+def test_msg_parse_only_headers():
+    """
+    Given:
+     - msg file.
+    When:
+     - parsing only the headers.
+    Then:
+     - Validate that the email was parsed.
+    """
+    test_path = 'parse_emails/tests/test_data/html_attachment.msg'
+
+    results = EmailParser(file_path=test_path, max_depth=3, parse_only_headers=True)
+    results.parse()
+
+    assert isinstance(results.parsed_email, dict)
+
+
+@pytest.mark.parametrize('headers, email_sender', [
+    ('To: <test@test.com> \nFrom: Services-Request September 06, 2024 <test@sender.com>',
+     ['<test@sender.com>']),
+    ('To: <test@test.com> \nFrom: Services-Request September 06 2024 <test@sender.com>',
+     ['Services-Request September 06 2024 <test@sender.com>']),
+    ('To: <test@test.com> \nFrom: "Services-Request September 06, 2024" <test@sender.com>',
+     ['Services-Request September 06, 2024 <test@sender.com>']),
+])
+def test_parse_email_headers(headers, email_sender):
+    """
+    Given:
+     - From header address with a display names with comma not wrapped in quotes.
+     - From header address with a display names without comma and not wrapped in quotes.
+     - From header address with a display names with comma wrapped in quotes.
+
+    When:
+     - parsing the headers.
+    Then:
+     - Validate that the email was parsed correctly.
+    """
+    parsed_headers = parse_email_headers(headers)
+
+    assert parsed_headers.get('From') == email_sender
+
+
+@pytest.mark.parametrize('file_type', ['application/pkcs7-mime', 'macintosh hfs', 'message/rfc822', 'multipart/alternative',
+                                       'multipart/mixed', 'multipart/related', 'multipart/signed', 'rfc 822 mail',
+                                       'smtp mail', 'utf-8 (with bom) text'])
+def test_eml_smtp_type(file_type):
     test_path = 'parse_emails/tests/test_data/smtp_email_type.eml'
-    test_type = 'SMTP mail, UTF-8 Unicode text, with CRLF terminators'
+    test_type = f'{file_type}, UTF-8 Unicode text, with CRLF terminators'
 
     results = EmailParser(file_path=test_path, max_depth=3, parse_only_headers=False, file_info=test_type)
     results.parse()
@@ -74,6 +117,17 @@ def test_eml_smtp_envelope_headers():
     assert results.parsed_email['Subject'] == 'Test Smtp Email'
 
 
+def test_eml_non_ascii():
+    test_path = 'parse_emails/tests/test_data/eml_non_ascii.eml'
+    test_type = 'news or mail text, Non-ISO extended-ASCII text, with CRLF line terminators'
+
+    results = EmailParser(file_path=test_path, max_depth=3, parse_only_headers=False, file_info=test_type)
+    results.parse()
+
+    assert isinstance(results.parsed_email, dict)
+    assert results.parsed_email['Subject'] == 'Test Non Ascii'
+
+
 # this is a test for another version of a multipart signed eml file
 def test_smime2():
 
@@ -95,8 +149,10 @@ def test_smime_entity_ascii_crlf():
     results = EmailParser(file_path=test_path, max_depth=3, parse_only_headers=False, file_info=test_type)
     results.parse()
 
-    assert isinstance(results.parsed_email, dict)
-    assert results.parsed_email['Subject'] == 'Testing Email Attachment'
+    assert isinstance(results.parsed_email, list)
+    assert results.parsed_email[0]['Subject'] is None
+    assert isinstance(results.parsed_email[0]['AttachmentsData'], list)
+    assert results.parsed_email[0]['AttachmentNames'] == ['smime.p7s', 'Attachment.eml']
 
 
 def test_eml_contains_eml():
@@ -148,6 +204,17 @@ def test_eml_contains_eml_depth():
 
 
 def test_eml_utf_text():
+    """
+    Given:
+        EML file containing a 'From' field in the following structure: "From: Test TEST, test<test@test.com>"
+    When:
+        parsing the email
+    Then:
+        assert it is parsed correctly.
+        parsed_email['From'] == 'test@test.com' and != 'Test, test@test.com'
+        TODO: When a fix is released: https://github.com/python/cpython/issues/107919
+              the conditional check 'if "@"' in the get_email_address function can be removed.
+    """
 
     test_path = 'parse_emails/tests/test_data/utf_8_email.eml'
     test_type = 'UTF-8 Unicode text, with very long lines, with CRLF line terminators'
@@ -157,6 +224,8 @@ def test_eml_utf_text():
 
     assert isinstance(results.parsed_email, dict)
     assert results.parsed_email['Subject'] == 'Test UTF Email'
+    assert results.parsed_email['From'] == 'test@test.com'
+    assert results.parsed_email['To'] == 'test@test.com'
 
 
 def test_eml_utf_text_special_chars():
@@ -324,8 +393,10 @@ def test_smime():
     results = EmailParser(file_path=test_path, max_depth=3, parse_only_headers=False, file_info=test_type)
     results.parse()
 
-    assert isinstance(results.parsed_email, dict)
-    assert results.parsed_email['Subject'] == 'Testing Email Attachment'
+    assert isinstance(results.parsed_email, list)
+    assert results.parsed_email[0]['Subject'] is None
+    assert isinstance(results.parsed_email[0]['AttachmentsData'], list)
+    assert results.parsed_email[0]['AttachmentNames'] == ['smime.p7s', 'Attachment.eml']
 
 
 def test_smime_msg():
@@ -356,7 +427,7 @@ def test_msg_headers_map():
 
 
 def test_unknown_file_info():
-    test_path = 'parse_emails/tests/test_data/smtp_email_type.eml'
+    test_path = 'parse_emails/tests/test_data/png.png'
     test_type = 'bad'
 
     try:
@@ -368,7 +439,6 @@ def test_unknown_file_info():
 
     assert gotexception
     assert 'Unknown file format:' in str(results)
-    assert 'smtp_email_type.eml' in str(results)
 
 
 def test_no_content_type_file():
@@ -600,6 +670,8 @@ def test_PtypString():
 def test_parse_body_with_russian_language():
     email_data, _, _ = handle_msg('parse_emails/tests/test_data/Phishing_TEST.msg', 'Phishing_TEST.msg')
     assert str(email_data['Text']).startswith('Уважаемые коллеги')
+    if isinstance(email_data['HTML'], bytes):
+        email_data['HTML'] = email_data['HTML'].decode()
     assert 'Уважаемые' in email_data['HTML']
 
 
@@ -674,8 +746,9 @@ def test_rtf_msg():
     test_path = 'parse_emails/tests/test_data/msg_with_rtf_compressed.msg'
     email_parser = EmailParser(file_path=test_path)
     results = email_parser.parse()
+    if isinstance(results['HTML'], bytes):
+        results['HTML'] = results['HTML'].decode()
     assert '<html xmlns:v="urn:schemas-microsoft-com:vml"' in results['HTML']
-    assert 'src="data:image/png;base64, ' in results['HTML']
 
 
 def test_eml_with_attachment_with_no_name():
@@ -695,25 +768,6 @@ def test_eml_with_attachment_with_no_name():
     assert 'VMail Enclosed for John Smith' in results['Subject']
 
 
-def test_embedding_image_into_html_of_eml():
-    """
-    Given:
-     - eml file containing images as CID (and contains HTML)
-
-    When:
-     - parsing eml file into email data.
-
-    Then:
-     - Validate that the src of the img tag contains the file in base64
-    """
-    test_path = 'parse_emails/tests/test_data/eml_contains_image_as_cid.eml'  # contains image.jpg as an attachment to the eml file.
-    email_parser = EmailParser(file_path=test_path)
-    results = email_parser.parse()
-    with open('parse_emails/tests/test_data/image.jpg', 'rb') as f:
-        base64_img = base64.b64encode(f.read()).decode('ascii')
-        assert f'src="data:image/jpeg;base64,{base64_img}"' in results['HTML']
-
-
 @pytest.mark.parametrize('data_value, data_type, expected_value',
                          [(b'\x01', '0x0002', 1),
                           (b'\x01', '0x0003', 1),
@@ -745,10 +799,33 @@ def test_parse_msg_contains_eml():
     assert results[1]['FileName'] == 'message.eml'
 
 
-def test_parse_eml_file_chinese_chars():
+@pytest.mark.parametrize(
+    'test_file_path, expected_chinese_str', [
+        (
+            'parse_emails/tests/test_data/chinese_gb2312_encoding.eml',
+            '你好，我是程序员，很高兴认识你'
+        ),
+        (
+            'parse_emails/tests/test_data/chinese_iso_2022_jp_encoding.eml',
+            '中文字'
+        ),
+        (
+            'parse_emails/tests/test_data/chinese_big5_encoding.eml',
+            '圖形碼'
+        ),
+        (
+            'parse_emails/tests/test_data/chinese_gbk_encoding.eml',
+            '你好，中国'
+        )
+    ]
+)
+def test_parse_eml_file_chinese_chars_encodings(test_file_path, expected_chinese_str):
     """
     Given:
-     - eml file that contains a text in chinese
+     - Case A: chinese eml file encoded in gb2312
+     - Case B: chinese eml file encoded in iso-2022-jp
+     - Case C: chinese eml file encoded in big5
+     - Case D: chinese eml file encoded in gbk
 
     When:
      - parsing the file.
@@ -756,8 +833,199 @@ def test_parse_eml_file_chinese_chars():
     Then:
      - make sure the chinese characters were decoded successfully.
     """
-    test_path = 'parse_emails/tests/test_data/chinese.eml'
-    email_parser = EmailParser(file_path=test_path)
+    email_parser = EmailParser(file_path=test_file_path)
     results = email_parser.parse()
 
-    assert results['Text'] == '你好，我是程序员，很高兴认识你'
+    assert results['Text'] == expected_chinese_str
+
+
+@pytest.mark.parametrize(
+    'test_file_path', [
+        (
+            'parse_emails/tests/test_data/smime_unicode_issue.p7m'
+        )
+    ]
+)
+def test_parse_p7m_file_with_unicode_spaces(test_file_path):
+    """
+    Given:
+     - Case A: .p7m file with unicode spaces.
+
+    When:
+     - parsing the file.
+
+    Then:
+     - Ensures no unicode spaces inside the text.
+    """
+    email_parser = EmailParser(file_path=test_file_path)
+    results = email_parser.parse()
+
+    assert "\u200a" not in results
+    assert "\u200d" not in results
+
+
+def test_parse_bcc_addresses_in_eml():
+    """
+    Given:
+     - eml file that contains bcc addresses
+    When:
+     - parsing the file.
+    Then:
+     - Validate that the bcc was returned.
+    """
+    test_path = 'parse_emails/tests/test_data/test bcc.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=3)
+    results = email_parser.parse()
+    assert results['BCC'] == 'test1@mail.com, test2@mail.com'
+
+
+def test_parse_eml_with_text_before_the_headers():
+    """
+    Given:
+     - eml file that starts with text that is not headers.
+    When:
+     - parsing the file.
+    Then:
+     - Validate that the text was removed and th efile is parsed.
+    """
+    test_path = 'parse_emails/tests/test_data/test_text_before_headers.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=3)
+    results = email_parser.parse()
+    assert len(results) == 15
+
+
+def test_multipart_defective():
+    """
+    Given:
+     - multipart eml that have defective boundary.
+    When:
+     - parsing the file.
+    Then:
+     - Validate that the eml parsed correctly.
+    """
+    test_path = 'parse_emails/tests/test_data/multipart-defective.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=2)
+    results = email_parser.parse()
+    assert results.get('Text')
+    assert results.get('HTML')
+
+
+def test_handle_eml_utf8_8bit():
+    """
+    Given:
+     - chinese eml file encoded in utf-8, 8bit
+
+    When:
+     - parsing the file.
+
+    Then:
+     - make sure the chinese characters were decoded successfully.
+    """
+    email_parser = EmailParser(file_path='parse_emails/tests/test_data/chinese_email_test.eml')
+    results = email_parser.parse()
+    expected_response = '这是一个示例邮件，用于演示指定的条件。\r\n它使用了 UTF-8 编码，可以支持多种语言的字符，包括中文。\r\n祝好，\r\n发件人'
+    assert results['Text'] == expected_response
+
+
+def test_eml_contains_image_name_with_Thai_characters():
+    """
+    Given:
+     - eml file contains image name with Thai characters
+
+    When:
+     - parsing the file.
+
+    Then:
+     - make sure the Thai character were decoded successfully.
+    """
+    test_path = 'parse_emails/tests/test_data/eml_contains_image_name_with_Thai_characters.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=2)
+    results = email_parser.parse()
+    assert results['Attachments'] == 'ผ้าห่ม06[4].jpg'
+
+
+def test_msg_contains_ascii_characters_with_null():
+    """
+    Given:
+     - msg file ASCII decoded contains null characters (\x00).
+
+    When:
+     - parsing the file.
+
+    Then:
+     - make sure the msg was correctly parsed.
+    """
+    test_path = 'parse_emails/tests/test_data/msg_with_null_characters_in_ascii_decode.msg'
+
+    email_parser = EmailParser(file_path=test_path)
+    results = email_parser.parse()
+    assert results['From'] == 'ZIEMSKI, Michal <michal.ziemski@wipo.int>'
+    assert results['Subject'] == 'RE: Test email for readpst and msg-extractor utility'
+
+
+def test_eml_contails_html_content_type():
+    test_path = 'parse_emails/tests/test_data/eml_contains_htm_content_type.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=2)
+    results = email_parser.parse()
+    assert len(results) == 15
+    assert results['HTML'] == '<html></html>'
+
+
+def test_handle_eml_unknown8bit():
+    """
+    Given:
+     - eml file header encoded in unknown-8bit in the name and in the email address
+
+    When:
+     - parsing the file.
+
+    Then:
+     - make sure the header were decoded successfully.
+    """
+    email_parser = EmailParser(file_path='parse_emails/tests/test_data/test-unknown-8bit.eml')
+    results = email_parser.parse()
+    assert results['From'] == 'test@test.com'
+    assert results['HeadersMap']['From'] == '"test" <test@test.com>'
+
+
+def test_multipart_eml_with_eml_attachment_containing_html_body():
+    """
+    Given:
+     - eml file with attached another eml file with text/html content.
+    When:
+     - parsing the file.
+    Then:
+     - make sure the msg was correctly parsed.
+    """
+    test_path = 'parse_emails/tests/test_data/multipart_with_eml_attachment_containing_html.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=2)
+    results = email_parser.parse()
+
+    assert len(results) == 2
+    assert results[0]["HTML"] == ""
+    assert results[0]["Attachments"] == "original_message.eml"
+    assert len(results[0]["AttachmentsData"]) > 0
+    assert results[1]["ParentFileName"] == "multipart_with_eml_attachment_containing_html.eml"
+
+
+def test_parse_attached_corrupted_eml():
+    """
+    Given:
+     - eml file with attached another corrupted eml file.
+    When:
+     - parsing the file.
+    Then:
+     - make sure it was correctly parsed.
+    """
+    test_path = 'parse_emails/tests/test_data/containing_corrupted_eml.eml'
+
+    email_parser = EmailParser(file_path=test_path, max_depth=2)
+    results = email_parser.parse()
+    assert len(results) == 2
+    assert results[0]['Subject'] == "Non-Delivery Report"
